@@ -58,34 +58,38 @@
 
 */
 
-#ifdef _WIN32
-typedef struct __stat64 xstat_t;
-#else
-typedef struct stat xstat_t;
-#endif
+#include <cstdint>  // uint64_t
+#include <cstdio>   // std::FILE, std::size_t
 
-uint64_t arch_get_memused();
-uint64_t arch_get_memtotal();
-long arch_get_cores();
-void arch_get_user_system_time(double * user_time, double * system_time);
-void arch_srandom();
-uint64_t arch_random();
-void * xmalloc(size_t size);
-void * xrealloc(void * ptr, size_t size);
-void xfree(void * ptr);
-
-int xfstat(int fd, xstat_t * buf);
-int xstat(const char * path, xstat_t  * buf);
-uint64_t xlseek(int fd, uint64_t offset, int whence);
-uint64_t xftello(FILE * stream);
-
-int xopen_read(const char * path);
-int xopen_write(const char * path);
-
-const char * xstrcasestr(const char * haystack, const char * needle);
 
 #ifdef _WIN32
-FARPROC arch_dlsym(HMODULE handle, const char * symbol);
+using xstat_t = struct __stat64;
 #else
-void * arch_dlsym(void * handle, const char * symbol);
+using xstat_t = struct stat;
+#endif
+
+auto arch_get_memused() -> uint64_t;
+auto arch_get_memtotal() -> uint64_t;
+auto arch_get_cores() -> long;
+auto arch_get_user_system_time(double * user_time, double * system_time) -> void;
+auto arch_srandom() -> void;
+auto arch_random() -> uint64_t;
+auto xmalloc(std::size_t size) -> void *;
+auto xrealloc(void * ptr, std::size_t size) -> void *;
+auto xfree(void * ptr) -> void;
+
+auto xfstat(int file_descriptor, xstat_t * buf) -> int;
+auto xstat(const char * path, xstat_t  * buf) -> int;
+auto xlseek(int file_descriptor, uint64_t offset, int whence) -> uint64_t;
+auto xftello(std::FILE * stream) -> uint64_t;
+
+auto xopen_read(const char * path) -> int;
+auto xopen_write(const char * path) -> int;
+
+auto xstrcasestr(const char * haystack, const char * needle) -> const char *;
+
+#ifdef _WIN32
+auto arch_dlsym(HMODULE handle, const char * symbol) -> FARPROC;
+#else
+auto arch_dlsym(void * handle, const char * symbol) -> void *;
 #endif
