@@ -59,11 +59,20 @@
 */
 
 #include "vsearch.h"
+#include "dynlibs.h"
+#include "maps.h"
+#include <cinttypes>  // macros PRIu64 and PRId64
+#include <climits>  // LONG_MAX
+#include <cstdint>  // int64_t, uint64_t
+#include <cstdio>  // std::FILE, std::fprintf, std::fclose, std::size_t, std::fread, std::fileno
+#include <cstdlib>  // std::exit, EXIT_FAILURE
+#include <cstring>  // std::memcpy, std::memcmp
+
 
 /* file compression and format detector */
 /* basic file buffering function for fastq and fastx parsers */
 
-#define FASTX_BUFFER_ALLOC 8192
+constexpr uint64_t fastx_buffer_alloc = 8192;
 
 #ifdef HAVE_BZLIB_H
 #define BZ_VERBOSE_0 0
@@ -75,24 +84,24 @@
 #define BZ_LESS_MEM 1  /* slower decompression but requires less memory */
 #endif
 
-#define FORMAT_PLAIN 1
-#define FORMAT_BZIP  2
-#define FORMAT_GZIP  3
+constexpr int format_plain = 1;
+constexpr int format_bzip = 2;
+constexpr int format_gzip = 3;
 
 static unsigned char MAGIC_GZIP[] = "\x1f\x8b";
 static unsigned char MAGIC_BZIP[] = "BZ";
 
 
-void buffer_init(struct fastx_buffer_s * buffer)
+auto buffer_init(struct fastx_buffer_s * buffer) -> void
 {
-  buffer->alloc = FASTX_BUFFER_ALLOC;
-  buffer->data = (char*) xmalloc(buffer->alloc);
+  buffer->alloc = fastx_buffer_alloc;
+  buffer->data = (char *) xmalloc(buffer->alloc);
   buffer->data[0] = 0;
   buffer->length = 0;
   buffer->position = 0;
 }
 
-void buffer_free(struct fastx_buffer_s * buffer)
+auto buffer_free(struct fastx_buffer_s * buffer) -> void
 {
   if (buffer->data)
     {
@@ -104,7 +113,7 @@ void buffer_free(struct fastx_buffer_s * buffer)
   buffer->position = 0;
 }
 
-void buffer_makespace(struct fastx_buffer_s * buffer, uint64_t x)
+auto buffer_makespace(struct fastx_buffer_s * buffer, uint64_t x) -> void
 {
   /* make sure there is space for x more chars in buffer */
 
@@ -113,17 +122,17 @@ void buffer_makespace(struct fastx_buffer_s * buffer, uint64_t x)
       /* alloc space for x more characters,
          but round up to nearest block size */
       buffer->alloc =
-        ((buffer->length + x + FASTX_BUFFER_ALLOC - 1) / FASTX_BUFFER_ALLOC)
-        * FASTX_BUFFER_ALLOC;
-      buffer->data = (char*) xrealloc(buffer->data, buffer->alloc);
+        ((buffer->length + x + fastx_buffer_alloc - 1) / fastx_buffer_alloc)
+        * fastx_buffer_alloc;
+      buffer->data = (char *) xrealloc(buffer->data, buffer->alloc);
     }
 }
 
-void buffer_extend(struct fastx_buffer_s * dest_buffer,
+auto buffer_extend(struct fastx_buffer_s * dest_buffer,
                    char * source_buf,
-                   uint64_t len)
+                   uint64_t len) -> void
 {
-  buffer_makespace(dest_buffer, len+1);
+  buffer_makespace(dest_buffer, len + 1);
   memcpy(dest_buffer->data + dest_buffer->length,
          source_buf,
          len);
@@ -131,7 +140,7 @@ void buffer_extend(struct fastx_buffer_s * dest_buffer,
   dest_buffer->data[dest_buffer->length] = 0;
 }
 
-void fastx_filter_header(fastx_handle h, bool truncateatspace)
+auto fastx_filter_header(fastx_handle h, bool truncateatspace) -> void
 {
   /* filter and truncate header */
 
@@ -140,8 +149,8 @@ void fastx_filter_header(fastx_handle h, bool truncateatspace)
 
   while (true)
     {
-      unsigned char c = *p++;
-      unsigned int m = char_header_action[c];
+      unsigned char const c = *p++;
+      unsigned int const m = char_header_action[c];
 
       switch(m)
         {
@@ -230,7 +239,7 @@ void fastx_filter_header(fastx_handle h, bool truncateatspace)
   h->header_buffer.length = q - h->header_buffer.data;
 }
 
-fastx_handle fastx_open(const char * filename)
+auto fastx_open(const char * filename) -> fastx_handle
 {
   auto * h = (fastx_handle) xmalloc(sizeof(struct fastx_s));
 
@@ -272,15 +281,15 @@ fastx_handle fastx_open(const char * filename)
 
   if (opt_gzip_decompress)
     {
-      h->format = FORMAT_GZIP;
+      h->format = format_gzip;
     }
   else if (opt_bzip2_decompress)
     {
-      h->format = FORMAT_BZIP;
+      h->format = format_bzip;
     }
   else if (h->is_pipe)
     {
-      h->format = FORMAT_PLAIN;
+      h->format = format_plain;
     }
   else
     {
@@ -290,19 +299,19 @@ fastx_handle fastx_open(const char * filename)
 
       unsigned char magic[2];
 
-      h->format = FORMAT_PLAIN;
+      h->format = format_plain;
 
-      size_t bytes_read = fread(&magic, 1, 2, h->fp);
+      size_t const bytes_read = fread(&magic, 1, 2, h->fp);
 
       if (bytes_read >= 2)
         {
           if (memcmp(magic, MAGIC_GZIP, 2) == 0)
             {
-              h->format = FORMAT_GZIP;
+              h->format = format_gzip;
             }
           else if (memcmp(magic, MAGIC_BZIP, 2) == 0)
             {
-              h->format = FORMAT_BZIP;
+              h->format = format_bzip;
             }
         }
       else
@@ -321,7 +330,7 @@ fastx_handle fastx_open(const char * filename)
         }
     }
 
-  if (h->format == FORMAT_GZIP)
+  if (h->format == format_gzip)
     {
       /* GZIP: Keep original file open, then open as gzipped file as well */
 #ifdef HAVE_ZLIB_H
@@ -339,7 +348,7 @@ fastx_handle fastx_open(const char * filename)
 #endif
     }
 
-  if (h->format == FORMAT_BZIP)
+  if (h->format == format_bzip)
     {
       /* BZIP2: Keep original file open, then open as bzipped file as well */
 #ifdef HAVE_BZLIB_H
@@ -367,7 +376,7 @@ fastx_handle fastx_open(const char * filename)
 
   /* start filling up file buffer */
 
-  uint64_t rest = fastx_file_fill_buffer(h);
+  uint64_t const rest = fastx_file_fill_buffer(h);
 
   /* examine first char and see if it starts with > or @ */
 
@@ -397,17 +406,17 @@ fastx_handle fastx_open(const char * filename)
 
           switch(h->format)
             {
-            case FORMAT_PLAIN:
+            case format_plain:
               break;
 
-            case FORMAT_GZIP:
+            case format_gzip:
 #ifdef HAVE_ZLIB_H
               (*gzclose_p)(h->fp_gz);
               h->fp_gz = nullptr;
               break;
 #endif
 
-            case FORMAT_BZIP:
+            case format_bzip:
 #ifdef HAVE_BZLIB_H
               (*BZ2_bzReadClose_p)(&bzError, h->fp_bz);
               h->fp_bz = nullptr;
@@ -449,7 +458,7 @@ fastx_handle fastx_open(const char * filename)
 
   h->stripped_all = 0;
 
-  for(uint64_t & i : h->stripped)
+  for (uint64_t & i : h->stripped)
     {
       i = 0;
     }
@@ -461,29 +470,29 @@ fastx_handle fastx_open(const char * filename)
   return h;
 }
 
-bool fastx_is_fastq(fastx_handle h)
+auto fastx_is_fastq(fastx_handle h) -> bool
 {
   return h->is_fastq || h->is_empty;
 }
 
-bool fastx_is_empty(fastx_handle h)
+auto fastx_is_empty(fastx_handle h) -> bool
 {
   return h->is_empty;
 }
 
-bool fastx_is_pipe(fastx_handle h)
+auto fastx_is_pipe(fastx_handle h) -> bool
 {
   return h->is_pipe;
 }
 
-void fastx_close(fastx_handle h)
+auto fastx_close(fastx_handle h) -> void
 {
   /* Warn about stripped chars */
 
   if (h->stripped_all)
     {
       fprintf(stderr, "WARNING: %" PRIu64 " invalid characters stripped from %s file:", h->stripped_all, (h->is_fastq ? "FASTQ" : "FASTA"));
-      for (int i=0; i<256;i++)
+      for (int i = 0; i < 256; i++)
         {
           if (h->stripped[i])
             {
@@ -509,22 +518,22 @@ void fastx_close(fastx_handle h)
     }
 
 #ifdef HAVE_BZLIB_H
-  int bz_error;
+  int bz_error = 0;
 #endif
 
   switch(h->format)
     {
-    case FORMAT_PLAIN:
+    case format_plain:
       break;
 
-    case FORMAT_GZIP:
+    case format_gzip:
 #ifdef HAVE_ZLIB_H
       (*gzclose_p)(h->fp_gz);
       h->fp_gz = nullptr;
       break;
 #endif
 
-    case FORMAT_BZIP:
+    case format_bzip:
 #ifdef HAVE_BZLIB_H
       (*BZ2_bzReadClose_p)(&bz_error, h->fp_bz);
       h->fp_bz = nullptr;
@@ -554,10 +563,10 @@ void fastx_close(fastx_handle h)
   h=nullptr;
 }
 
-uint64_t fastx_file_fill_buffer(fastx_handle h)
+auto fastx_file_fill_buffer(fastx_handle h) -> uint64_t
 {
   /* read more data if necessary */
-  uint64_t rest = h->file_buffer.length - h->file_buffer.position;
+  uint64_t const rest = h->file_buffer.length - h->file_buffer.position;
 
   if (rest > 0)
     {
@@ -583,7 +592,7 @@ uint64_t fastx_file_fill_buffer(fastx_handle h)
 
       switch(h->format)
         {
-        case FORMAT_PLAIN:
+        case format_plain:
           bytes_read = fread(h->file_buffer.data
                              + h->file_buffer.position,
                              1,
@@ -591,7 +600,7 @@ uint64_t fastx_file_fill_buffer(fastx_handle h)
                              h->fp);
           break;
 
-        case FORMAT_GZIP:
+        case format_gzip:
 #ifdef HAVE_ZLIB_H
           bytes_read = (*gzread_p)(h->fp_gz,
                                    h->file_buffer.data
@@ -604,7 +613,7 @@ uint64_t fastx_file_fill_buffer(fastx_handle h)
           break;
 #endif
 
-        case FORMAT_BZIP:
+        case format_bzip:
 #ifdef HAVE_BZLIB_H
           bytes_read = (*BZ2_bzRead_p)(& bzError,
                                        h->fp_bz,
@@ -628,10 +637,10 @@ uint64_t fastx_file_fill_buffer(fastx_handle h)
       if (! h->is_pipe)
         {
 #ifdef HAVE_ZLIB_H
-          if (h->format == FORMAT_GZIP)
+          if (h->format == format_gzip)
             {
               /* Circumvent the missing gzoffset function in zlib 1.2.3 and earlier */
-              int fd = dup(fileno(h->fp));
+              int const fd = dup(fileno(h->fp));
               h->file_position = xlseek(fd, 0, SEEK_CUR);
               close(fd);
             }
@@ -647,9 +656,9 @@ uint64_t fastx_file_fill_buffer(fastx_handle h)
     }
 }
 
-bool fastx_next(fastx_handle h,
+auto fastx_next(fastx_handle h,
                 bool truncateatspace,
-                const unsigned char * char_mapping)
+                const unsigned char * char_mapping) -> bool
 {
   if (h->is_fastq)
     {
@@ -661,7 +670,7 @@ bool fastx_next(fastx_handle h,
     }
 }
 
-uint64_t fastx_get_position(fastx_handle h)
+auto fastx_get_position(fastx_handle h) -> uint64_t
 {
   if (h->is_fastq)
     {
@@ -674,7 +683,7 @@ uint64_t fastx_get_position(fastx_handle h)
 }
 
 
-uint64_t fastx_get_size(fastx_handle h)
+auto fastx_get_size(fastx_handle h) -> uint64_t
 {
   if (h->is_fastq)
     {
@@ -687,7 +696,7 @@ uint64_t fastx_get_size(fastx_handle h)
 }
 
 
-uint64_t fastx_get_lineno(fastx_handle h)
+auto fastx_get_lineno(fastx_handle h) -> uint64_t
 {
   if (h->is_fastq)
     {
@@ -700,7 +709,7 @@ uint64_t fastx_get_lineno(fastx_handle h)
 }
 
 
-uint64_t fastx_get_seqno(fastx_handle h)
+auto fastx_get_seqno(fastx_handle h) -> uint64_t
 {
   if (h->is_fastq)
     {
@@ -712,7 +721,7 @@ uint64_t fastx_get_seqno(fastx_handle h)
     }
 }
 
-char * fastx_get_header(fastx_handle h)
+auto fastx_get_header(fastx_handle h) -> char *
 {
   if (h->is_fastq)
     {
@@ -724,7 +733,7 @@ char * fastx_get_header(fastx_handle h)
     }
 }
 
-char * fastx_get_sequence(fastx_handle h)
+auto fastx_get_sequence(fastx_handle h) -> char *
 {
   if (h->is_fastq)
     {
@@ -736,7 +745,7 @@ char * fastx_get_sequence(fastx_handle h)
     }
 }
 
-uint64_t fastx_get_header_length(fastx_handle h)
+auto fastx_get_header_length(fastx_handle h) -> uint64_t
 {
   if (h->is_fastq)
     {
@@ -748,7 +757,7 @@ uint64_t fastx_get_header_length(fastx_handle h)
     }
 }
 
-uint64_t fastx_get_sequence_length(fastx_handle h)
+auto fastx_get_sequence_length(fastx_handle h) -> uint64_t
 {
   if (h->is_fastq)
     {
@@ -761,7 +770,7 @@ uint64_t fastx_get_sequence_length(fastx_handle h)
 }
 
 
-char * fastx_get_quality(fastx_handle h)
+auto fastx_get_quality(fastx_handle h) -> char *
 {
   if (h->is_fastq)
     {
@@ -773,7 +782,7 @@ char * fastx_get_quality(fastx_handle h)
     }
 }
 
-int64_t fastx_get_abundance(fastx_handle h)
+auto fastx_get_abundance(fastx_handle h) -> int64_t
 {
   if (h->is_fastq)
     {
