@@ -58,6 +58,11 @@
 
 */
 
+#include <cstdint>  // int64_t
+#include <cstdio>  // std::FILE
+#include <vector>
+
+
 struct msa_target_s
 {
   int seqno;
@@ -65,7 +70,7 @@ struct msa_target_s
   int strand;
 };
 
-void msa(FILE * fp_msaout, FILE * fp_consout, FILE * fp_profile,
+auto msa(std::FILE * fp_msaout, std::FILE * fp_consout, std::FILE * fp_profile,
          int cluster,
-         int target_count, struct msa_target_s * target_list,
-         int64_t totalabundance);
+         int target_count, std::vector<struct msa_target_s> const & target_list_v,
+         int64_t totalabundance) -> void;
