@@ -58,4 +58,4 @@
 
 */
 
-void orient();
+auto orient() -> void;
