@@ -236,6 +236,7 @@ extern bool opt_fastq_nostagger;
 extern bool opt_gzip_decompress;
 extern bool opt_label_substr_match;
 extern bool opt_lengthout;
+extern bool opt_n_mismatch;
 extern bool opt_no_progress;
 extern bool opt_quiet;
 extern bool opt_relabel_keep;
@@ -345,6 +346,7 @@ extern double opt_fastq_maxdiffpct;
 extern double opt_fastq_maxee;
 extern double opt_fastq_maxee_rate;
 extern double opt_fastq_truncee;
+extern double opt_fastq_truncee_rate;
 extern double opt_id;
 extern double opt_lca_cutoff;
 extern double opt_max_unmasked_pct;
@@ -403,6 +405,7 @@ extern int64_t opt_fastq_maxns;
 extern int64_t opt_fastq_minlen;
 extern int64_t opt_fastq_minmergelen;
 extern int64_t opt_fastq_minovlen;
+extern int64_t opt_fastq_minqual;
 extern int64_t opt_fastq_qmax;
 extern int64_t opt_fastq_qmaxout;
 extern int64_t opt_fastq_qmin;
